indexing
   title:       "TIMETOOLS"
   cluster:     "timedate [timedate]";
   project:     "The Universal Time & Date UTDC";
   copyright:   "SIG Computer, 1995";
   author:      "Alexei Simonov (alex@eiffel.ru)";
   approved_by: "Frieder Monninger";
   original:    22,Sep,95
   version:     1.0
   done_at:     "SIG Computer (info@eiffel.ru)";
   license:     "GNU Library GPL - see included readme file";
   extrnl_name: "timetool.e";

class TIME_TOOLS

   feature -- Queries

      -- Days
      monday      : INTEGER is 1;
      tuesday     : INTEGER is 2;
      wednesday   : INTEGER is 3;
      thursday    : INTEGER is 4;
      friday      : INTEGER is 5;
      saturday    : INTEGER is 6;
      sunday      : INTEGER is 7;

      -- monthes
      january     : INTEGER is  1;
      february    : INTEGER is  2;
      march       : INTEGER is  3;
      april       : INTEGER is  4;
      may         : INTEGER is  5;
      june        : INTEGER is  6;
      july        : INTEGER is  7;
      august      : INTEGER is  8;
      september   : INTEGER is  9;
      october     : INTEGER is 10;
      november    : INTEGER is 11;
      december    : INTEGER is 12;

      -- constants
      zero                   : INTEGER is            0;
      one_day                : INTEGER is            1;
      one_month              : INTEGER is            1;
      one_year               : INTEGER is            1;
      one_millisecond        : INTEGER is            1;
      second_month           : INTEGER is            2;
      leap_year              : INTEGER is            4;
      leap_const1            : INTEGER is          100;
      leap_const2            : INTEGER is          400;
      months_in_year         : INTEGER is           12;
      days_in_week           : INTEGER is            7;
      days_in_month_d        : INTEGER is           30;
      days_in_year           : INTEGER is          365;
      days_in_year_d         : INTEGER is          360;
      days_4_year            : INTEGER is        1_461;
      days_100_year          : INTEGER is       36_524;
      days_400_year          : INTEGER is      146_097;
      hours_in_day           : INTEGER is           24;
      minutes_in_hour        : INTEGER is           60;
      minutes_in_day         : INTEGER is        1_440;
      seconds_in_minute      : INTEGER is           60;
      seconds_in_hour        : INTEGER is        3_600;
      seconds_in_day         : INTEGER is       86_400;
      seconds_in_month       : INTEGER is    2_592_000;
      seconds_in_year        : INTEGER is   31_104_000;
      one                    : REAL    is          1.0;
      milliseconds_in_second : INTEGER is        1_000;
      milliseconds_in_day    : INTEGER is   86_400_000;

      valid_time ( hour, minute : INTEGER; second : REAL ) : BOOLEAN is
         do
            Result := second >= zero and then
                      second <  seconds_in_minute and then
                      minute >= zero and then
                      minute <  minutes_in_hour and then
                      hour   >= zero and then
                      hour   <  hours_in_day
         end -- valid_time

      valid_date ( year, month, day : INTEGER ) : BOOLEAN is
         local
            c : BOOLEAN;
         do
            if is_leap_year( year ) = true  and month = second_month then
               c := day <= ( day_in_months.item( month ) + one_month )
            else
               c := day <= day_in_months.item( month )
            end -- if
            Result := day   >  zero and then
                      c     =  true and then
                      month >  zero and then
                      month <= months_in_year and then
                      year  >  zero;

         end -- valid_date

      is_leap_year ( yr : INTEGER ) : BOOLEAN is
         do
            if ( yr \\ leap_year ) = zero and
               ( yr \\ leap_const1 ) /= zero then
               Result := true;
            elseif ( yr \\ leap_year ) = zero and
                   ( yr \\ leap_const2 ) = zero then
               Result := true;
            else
               Result := false;
            end -- if
         end -- is_leap_year

      day_name ( n : INTEGER ) : STRING is
         require
            non_valid_day_name : n >= one_day and then n <= days_in_week
         do
            Result := days.item( n );
         end -- day_name

      month_name ( n : INTEGER ) : STRING is
         require
            non_valid_month_name : n >= one_month and then n <= months_in_year
         do
            Result := monthes.item( n );
         end -- month_name

      days_in_month ( n : INTEGER ) : INTEGER is
         require
            non_valid_day_name : n >= one_month and then n <= months_in_year
         do
            Result := day_in_months.item( n );
         end -- day_name

      days : ARRAY [ STRING ] is
         once
            Result := <<
               "MONDAY",
               "TUESDAY",
               "WEDNESDAY",
               "THURSDAY",
               "FRIDAY",
               "SATURDAY",
               "SUNDAY"
            >>;
         end -- days

      monthes : ARRAY [ STRING ] is
         once
            Result := <<
               "JANUARY",
               "FEBRUARY",
               "MARCH",
               "APRIL",
               "MAY",
               "JUNE",
               "JULY",
               "AUGUST",
               "SEPTEMBER",
               "OCTOBER",
               "NOVEMBER",
               "DECEMBER"
            >>;
         end -- monthes

      day_in_months : ARRAY [ INTEGER ] is
         once
            Result := <<
               31,
               28,
               31,
               30,
               31,
               30,
               31,
               31,
               30,
               31,
               30,
               31
            >>;
         end -- days_in_months

      days_in_months : ARRAY [ INTEGER ] is
         once
            Result := <<
                 0,
                31,
                59,
                90,
               120,
               151,
               181,
               212,
               243,
               273,
               304,
               334,
               365
            >>;
          end -- days_in_months

      day_zero : TIMESTAMP is
         once
            !!Result.set ( 1, 1, 1, 0, 0, 0.0 );
         end -- day_zero

end -- class TIME_TOOLS
